@@ -39,7 +39,7 @@ def setup_config(cli_opts):
 
     # load up credentials file if present
     try:
-        home = os.environ.get('HOME', None)
+        home = os.path.expanduser('~')
         credential_file = os.path.join(home, '.anchore', 'credentials.yaml')
         if os.path.exists(credential_file):
             ydata = {}
@@ -703,7 +703,15 @@ def format_output(config, op, params, payload):
                             code_version = service_record['service_detail']['version']
                         except:
                             pass
-                    
+                        
+                    # override with any discovered API service that is up
+                    try:
+                        if service_record['servicename'] == 'apiext' and service_status == 'up':
+                            api_code_version = service_record['service_detail']['version']
+                            code_version = api_code_version
+                    except:
+                        pass
+                        
                 for k in outlist:
                     obuf = obuf + k + "\n"
                 obuf = obuf + "\n"
@@ -744,7 +752,10 @@ def format_output(config, op, params, payload):
                 for el in payload:
                     feed = el.get('name', "N/A")
                     for gel in el['groups']:
-                        t.add_row([feed, gel.get('name', "N/A"), gel.get('last_sync', "N/A"), gel.get('record_count', "N/A")])
+                        last_sync = gel.get('last_sync', None)
+                        if not last_sync:
+                            last_sync = "pending"
+                        t.add_row([feed, gel.get('name', "N/A"), last_sync, gel.get('record_count', "N/A")])
                 ret = t.get_string(sortby='Feed')+"\n"
             except Exception as err:
                 raise err
@@ -761,13 +772,23 @@ def format_output(config, op, params, payload):
                         t.add_row(row)
                 ret = t.get_string(sortby='Feed')
         elif op == 'event_list':
-            header = ['Timestamp', 'Level', 'Service', 'Host', 'Event', 'ID']
+            header = ['Timestamp', 'Level', 'Event', 'Resource', 'ID']
             t = PrettyTable(header)
             t.set_style(PLAIN_COLUMNS)
             t.align = 'l'
             for event_res in payload['results']:
                 event = event_res['event']
-                row = [event['timestamp'], event['level'], event['source']['servicename'], event['source']['hostid'], event['type'], event_res['generated_uuid']]
+                row = [event['timestamp'], event['level'], event['type'], event['resource'].get('id'), event_res['generated_uuid']]
+                t.add_row(row)
+            ret = t.get_string()
+        elif op == 'event_list_full':
+            header = ['Timestamp', 'Level', 'Event', 'ResourceType', 'Resource', 'Service', 'Host', 'ID']
+            t = PrettyTable(header)
+            t.set_style(PLAIN_COLUMNS)
+            t.align = 'l'
+            for event_res in payload['results']:
+                event = event_res['event']
+                row = [event['timestamp'], event['level'], event['type'], event['resource'].get('type'), event['resource'].get('id'), event['source']['servicename'], event['source']['hostid'], event_res['generated_uuid']]
                 t.add_row(row)
             ret = t.get_string()
         elif op == 'event_get':
