@@ -14,7 +14,7 @@ def make_client_result(response, raw=False):
     try:
         ret['httpcode'] = response.status_code
 
-        if response.status_code == 200:
+        if response.status_code in range(200, 299):
             ret['success'] = True
             if raw == True:
                 ret['payload'] = response.text
@@ -34,6 +34,9 @@ def make_client_result(response, raw=False):
                     ret['error'] = json.loads(response.text)
                 except:
                     ret['error'] = response.text
+
+            if not ret.get('error', None) and response.status_code in [401]:
+                ret['error'] = "Unauthorized - please check your username/password"
     except Exception as err:
         raise err
 
